local XarUI = LibStub('AceAddon-3.0'):NewAddon('XarUI', 'AceConsole-3.0', 'AceEvent-3.0')

local   UnitIsPlayer, UnitIsConnected, UnitClass, RAID_CLASS_COLORS, UnitAffectingCombat =
        UnitIsPlayer, UnitIsConnected, UnitClass, RAID_CLASS_COLORS, UnitAffectingCombat
local sb = _G.GameTooltipStatusBar
local _, class, c, db


local function colorStatusbar(statusbar, unit)
    if not db.profile.classColors then return end

    if UnitIsPlayer(unit) and UnitIsConnected(unit) and UnitClass(unit) then
        if unit == statusbar.unit or statusbar == sb then
            _, class = UnitClass(unit)
            c = RAID_CLASS_COLORS[class]
            statusbar:SetStatusBarColor(c.r, c.g, c.b)
        end
    end
end

hooksecurefunc('UnitFrameHealthBar_Update', colorStatusbar)
hooksecurefunc('HealthBar_OnValueChanged', function(self)
    colorStatusbar(self, self.unit)
end)

function XarUI:UPDATE_MOUSEOVER_UNIT()
    colorStatusbar(sb, 'mouseover')
end

local trackingHook = true
local function setTrackingBarVisibility()
    if InCombatLockdown() then return end

    if db.profile.hideTrackingBars then
        MainMenuBar:SetUserPlaced(true)

        MainMenuBar:ClearAllPoints()
        MainMenuBar:SetPoint('BOTTOM', UIParent, 0, 0)

        MainMenuBarArtFrame.LeftEndCap:ClearAllPoints()
        MainMenuBarArtFrame.RightEndCap:ClearAllPoints()
        MainMenuBarArtFrame.LeftEndCap:SetPoint('BOTTOMLEFT', MainMenuBar, -98, 0)
        MainMenuBarArtFrame.RightEndCap:SetPoint('BOTTOMRIGHT', MainMenuBar, 98, 0)
    else
        MainMenuBar:SetUserPlaced(false)
        trackingHook = false
        MainMenuBar:SetPositionForStatusBars()
        trackingHook = true
    end
end
hooksecurefunc(MainMenuBar, 'ChangeMenuBarSizeAndPosition', setTrackingBarVisibility)
hooksecurefunc(MainMenuBar, 'SetPositionForStatusBars', function()
    if trackingHook then
        setTrackingBarVisibility()
    end
end)

local function setEndCapVisibility()
    local hide = db.profile.hideEndCaps
    MainMenuBarArtFrame.RightEndCap:SetShown(not hide)
    MainMenuBarArtFrame.LeftEndCap:SetShown(not hide)
end

local feedbackText = PlayerFrame:CreateFontString(nil, 'OVERLAY', 'NumberFontNormalHuge')
local function setFeedbackVisibility()
    if db.profile.hideFeedbackText then
        PlayerFrame.feedbackText = feedbackText
        PlayerFrame.feedbackStartTime = 0
        PetFrame.feedbackText = feedbackText
        PetFrame.feedbackStartTime = 0

        PlayerHitIndicator:Hide()
        PetHitIndicator:Hide()
    else
        local time = GetTime()

        PlayerFrame.feedbackText = PlayerHitIndicator
        PlayerFrame.feedbackStartTime = time
        PetFrame.feedbackText = PetHitIndicator
        PetFrame.feedbackStartTime = time
    end
end

local dampeningFrame = CreateFrame('Frame', nil , UIParent)
dampeningFrame:SetSize(200, 12)
dampeningFrame:SetPoint('TOP', UIWidgetTopCenterContainerFrame, 'BOTTOM', 0, -2)
dampeningFrame.text = dampeningFrame:CreateFontString(nil, 'BACKGROUND')
dampeningFrame.text:SetFontObject(GameFontNormalSmall)
dampeningFrame.text:SetAllPoints()
dampeningFrame.timeSinceLastUpdate = 0
local updateInterval = 5
local dampeningText = GetSpellInfo(110310)
dampeningFrame:SetScript('OnUpdate', function(self, elapsed)
    self.timeSinceLastUpdate = self.timeSinceLastUpdate + elapsed

    if self.timeSinceLastUpdate > updateInterval then
        self.text:SetText(dampeningText..': '..C_Commentator.GetDampeningPercent()..'%')
        self.timeSinceLastUpdate = 0
    end
end)

local targetCombatFrame = CreateFrame('Frame', nil , TargetFrame)
targetCombatFrame:SetPoint('LEFT', TargetFrame, 'RIGHT', -20, 0)
targetCombatFrame:SetSize(26,26)
targetCombatFrame.icon = targetCombatFrame:CreateTexture(nil, 'BORDER')
targetCombatFrame.icon:SetAllPoints()
targetCombatFrame.icon:SetTexture([[Interface\Icons\ABILITY_DUALWIELD]])
targetCombatFrame:Hide()

local focusCombatFrame = CreateFrame('Frame', nil , FocusFrame)
focusCombatFrame:SetPoint('LEFT', FocusFrame, 'RIGHT', -20, 0)
focusCombatFrame:SetSize(26,26)
focusCombatFrame.icon = focusCombatFrame:CreateTexture(nil, 'BORDER')
focusCombatFrame.icon:SetAllPoints()
focusCombatFrame.icon:SetTexture([[Interface\Icons\ABILITY_DUALWIELD]])
focusCombatFrame:Hide()

local combatFrame = CreateFrame('Frame', nil , UIParent)
combatFrame.timeSinceLastUpdate = 0
local combatInterval = 0.1
local function combatUpdate(self, elapsed)
    self.timeSinceLastUpdate = self.timeSinceLastUpdate + elapsed

    if self.timeSinceLastUpdate > combatInterval then
        targetCombatFrame:SetShown(UnitAffectingCombat('target'))
        focusCombatFrame:SetShown(UnitAffectingCombat('focus'))
        self.timeSinceLastUpdate = 0
    end
end

local function setCombatTracking()
    if db.profile.trackCombat then
        combatFrame:SetScript('OnUpdate', combatUpdate)
    else
        combatFrame:SetScript('OnUpdate', nil)
        targetCombatFrame:Hide()
        focusCombatFrame:Hide()
    end
end

function XarUI:PLAYER_ENTERING_WORLD()
    local _, instanceType = IsInInstance()
    dampeningFrame:SetShown(instanceType == 'arena')
end

local function moveButtons()
    if db.profile.moveLeftButton12 then
        MultiBarLeftButton12:ClearAllPoints()
        MultiBarLeftButton12:SetPoint('LEFT', MultiBarBottomLeftButton12, 'RIGHT', 6, 0)
    else
        MultiBarLeftButton12:ClearAllPoints()
        MultiBarLeftButton12:SetPoint('TOP', MultiBarLeftButton11, 'BOTTOM', 0, -6)
    end

    if db.profile.moveRightButton12 then
        MultiBarRightButton12:ClearAllPoints()
        MultiBarRightButton12:SetPoint('LEFT', MultiBarBottomLeftButton12, 'TOP', 5, 25)
        MultiBarRightButton12:SetScale(0.9)
    else
        MultiBarRightButton12:ClearAllPoints()
        MultiBarRightButton12:SetPoint('TOP', MultiBarRightButton11, 'BOTTOM', 0, -6)
        MultiBarRightButton12:SetScale(1)
    end
end


local function refreshProfile()
    setEndCapVisibility()
    setTrackingBarVisibility()
    setFeedbackVisibility()
    setCombatTracking()
    moveButtons()
end

local function validateCombat()
    if InCombatLockdown() then
        return 'Must leave combat first.'
    end

    return true
end

local defaults = {
    profile = {
        classColors = true,
        hideTrackingBars = true,
        hideEndCaps = true,
        hideFeedbackText = false,
        trackCombat = true,
        moveLeftButton12 = true,
        moveRightButton12 = true,
    },
}

local options = {
    name = "|cff00c0ffXarUI|r",
    type = 'group',
    childGroups = 'tab',
    validate = validateCombat,
    args = {
        general = {
            name = 'General',
            type = 'group',
            order = 1,
            get = function(info) return db.profile[info[#info]] end,
            set = function(info, val) db.profile[info[#info]] = val end,
            args = {
                classColors = {
                    name = 'Class-colored health bars',
                    desc = 'Changes take effect when health is lost/gained',
                    type = 'toggle',
                    order = 1,
                    width = 'full',
                    validate = function() return true end,
                },
                hideTrackingBars = {
                    name = 'Hide tracking bars',
                    type = 'toggle',
                    order = 2,
                    width = 'full',
                    set = function(info, val)
                        db.profile[info[#info]] = val
                        setTrackingBarVisibility()
                    end,
                },
                hideEndCaps = {
                    name = 'Hide action bar end caps',
                    type = 'toggle',
                    order = 3,
                    width = 'full',
                    set = function(info, val)
                        db.profile[info[#info]] = val
                        setEndCapVisibility()
                    end,
                },
                hideFeedbackText = {
                    name = 'Hide feedback text',
                    desc = 'Damage & healing text on player portrait',
                    type = 'toggle',
                    order = 4,
                    width = 'full',
                    validate = function() return true end,
                    set = function(info, val)
                        db.profile[info[#info]] = val
                        setFeedbackVisibility()
                    end,
                },
                trackCombat = {
                    name = 'Combat tracking',
                    desc = 'Shows icon when target/focus is in combat',
                    type = 'toggle',
                    order = 5,
                    width = 'full',
                    validate = function() return true end,
                    set = function(info, val)
                        db.profile[info[#info]] = val
                        setCombatTracking()
                    end,
                },
                moveRightButton12 = {
                    name = 'Move right-button-12',
                    desc = 'Must have right bar enabled',
                    type = 'toggle',
                    order = 6,
                    width = 'full',
                    set = function(info, val)
                        db.profile[info[#info]] = val
                        moveButtons()
                    end,
                },
                moveLeftButton12 = {
                    name = 'Move left-button-12',
                    desc = 'Must have right bar 2 enabled',
                    type = 'toggle',
                    order = 7,
                    width = 'full',
                    set = function(info, val)
                        db.profile[info[#info]] = val
                        moveButtons()
                    end,
                },
            },
        },
    },
}

function XarUI:OnInitialize()
    db = LibStub('AceDB-3.0'):New('XarUIDB', defaults, true)
    db.RegisterCallback(self, 'OnProfileChanged', refreshProfile)
    db.RegisterCallback(self, 'OnProfileCopied', refreshProfile)
    db.RegisterCallback(self, 'OnProfileReset', refreshProfile)
    options.args.profile = LibStub('AceDBOptions-3.0'):GetOptionsTable(db)
    LibStub('AceConfig-3.0'):RegisterOptionsTable('XarUI', options)
    LibStub('AceConfigDialog-3.0'):AddToBlizOptions('XarUI', '|cff00c0ffXarUI|r')
    LibStub('AceConfigDialog-3.0'):SetDefaultSize('XarUI', 400, 350)
    self:RegisterChatCommand('xarui', function() LibStub('AceConfigDialog-3.0'):Open('XarUI') end)
    self:RegisterChatCommand('xar', function() LibStub('AceConfigDialog-3.0'):Open('XarUI') end)
    self:RegisterChatCommand('rl', ReloadUI)

    -- Class colors on tooltip health bar
    -- self:RegisterEvent('UPDATE_MOUSEOVER_UNIT')
    self:RegisterEvent('PLAYER_ENTERING_WORLD')

    MainMenuBar:SetMovable(true)

    refreshProfile()

    -- Hide minimap zoom icons & enable scroll wheel zooming
    MinimapZoomIn:Hide()
    MinimapZoomOut:Hide()

    Minimap:EnableMouseWheel(true)
    Minimap:SetScript('OnMouseWheel', function(self, arg1)
        if arg1 > 0 then
            Minimap_ZoomIn()
        else
            Minimap_ZoomOut()
        end
    end)

    -- Hide shading on LoC frame
    LossOfControlFrame.blackBg:SetAlpha(0)
    LossOfControlFrame.RedLineTop:SetAlpha(0)
    LossOfControlFrame.RedLineBottom:SetAlpha(0)

    -- Copied from original XarUI

    UIErrorsFrame:Hide()
    MicroButtonAndBagsBar:Hide()
    QuickJoinToastButton:Hide()
    ChatFrameChannelButton:Hide()

    -- StanceButton1:SetScale(1.15)

    -- CRFSort_Group = function(t1, t2)
    --     if UnitIsUnit(t1,'player') then
    --         return false
    --     elseif UnitIsUnit(t2,'player') then
    --         return true
    --     else
    --         return t1 < t2
    --     end
    -- end
    -- CompactRaidFrameContainer.flowSortFunc=CRFSort_Group
end
